import re
import os

reExtentFileName = re.compile('^e\d+$', re.IGNORECASE)

def unPageAttachments(doc):

  """
  Special processing is required for OLE embedded docs.  Notes
  apparently stores OLE embedded docs as a series of 8K pages, with a
  separate 'extent' file that expresses in what order the pages need
  to be concatenated to reconstruct the original doc.
  """

  # NOTE: I could not find any Notes C API docs covering the details
  # of how paged OLE attachments work.  So far this has been inferred
  # from one customer context!

  if '$FILE' in doc:

    # We record each page file we've concatenated in this dict so on
    # second pass we can remove those files:
    toRemove = {}

    fileNameList = doc['$FILE']
    newFileNameList = []
    for fileName in fileNameList:

      # Remove "file://" leader:
      justFileName = fileName[7:]
      if justFileName.startswith('./'):
        justFileName = justFileName[2:]
      dirName, fileNamePart = os.path.split(justFileName)

      if fileNamePart.lower() in toRemove:
        pass
      elif reExtentFileName.match(fileNamePart) is not None:

        # Read file contents:
        data = open(justFileName, 'rb').read()
        header = data[:4]

        # Verify header looks right:
        if header == '\001\000\000\000':

          l = data[4:].split('\000')
          l = [elem for elem in l if elem.strip() != '']

          # NOTE: sometimes the first element *seems* to be # pages --
          # we could use this to do additional checking to confirm we
          # _really_ are looking at a OLE paging file; other times
          # this count is missing!

          # Just concatenate the pages, in order:
          fileNameOut = '%s/OLE%s.doc' % (dirName, fileNamePart[1:])
          print 'NOTE: OLE: create %s from %d pages' % (fileNameOut, len(l))
          
          fOut = open(fileNameOut, 'wb')
          first = 1
          for pageFileName in l:
            toRemove[pageFileName.lower()] = 1
            fullPageFileName = '%s/%s' % (dirName, pageFileName)
            if not os.path.exists(fullPageFileName):
              # Only warn if it's not the first file, since sometimes this is a length indicator:
              if not first:
                print 'WARNING: OLE: referenced page file "%s" is not found; skipping' % fullPageFileName
              continue
            first = 0
            try:
              data = open(fullPageFileName, 'rb').read()
              fOut.write(data)
            except IOError:
              print 'WARNING: OLE: referenced page file "%s" IOERROR; skipping' % fullPageFileName

              continue
            # Remove page file from filesystem:
            os.remove(fullPageFileName)
            
          fOut.close()

          # Remove extent file from filesystem:
          os.remove(justFileName)
          newFileNameList.append('file://./%s' % fileNameOut)
          
        else:
          # Looked like an OLE extent file, but apparently was not:
          print 'WARNING: OLE: extent file lookalike "%s"' % fileName
          newFileNameList.append(fileName)
      else:
        # Must be another normal (non-OLE) file attachment.  It could
        # also be page file, in which case we'll remove it on second
        # pass below:
        newFileNameList.append(fileName)

    # Second pass to remove page files:
    i = 0
    while i < len(newFileNameList):
      justFileName = newFileNameList[i][7:]
      dirName, fileNamePart = os.path.split(justFileName)
      if fileNamePart.lower() in toRemove:
        del newFileNameList[i]
      else:
        i += 1

    doc['$FILE'] = newFileNameList

